"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ImportIdentifierCollection = void 0;
const tslib_1 = require("tslib");
const typescript_1 = tslib_1.__importDefault(require("typescript"));
const dumpNode_cjs_1 = require("./dumpNode.cjs");
const util_cjs_1 = require("./util.cjs");
/** @internal */
class ImportIdentifierCollection {
    constructor(checker) {
        Object.defineProperty(this, "checker", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: checker
        });
        Object.defineProperty(this, "_length", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: 0
        });
        Object.defineProperty(this, "idRegistry", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: new Map()
        });
        Object.defineProperty(this, "unusedIds", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: new Set()
        });
    }
    get length() {
        return this._length;
    }
    add(node) {
        if (this._addNamespaceImport(node)) {
            return;
        }
        this._addNamedImport(node);
        this._addDefaultImport(node);
    }
    /** import def from */
    _addDefaultImport(node) {
        var _a;
        const name = (_a = node.importClause) === null || _a === void 0 ? void 0 : _a.name;
        if (!name) {
            return false;
        }
        this.pushItem(node, name, 'default');
        return true;
    }
    /** import {a as b, c} from */
    _addNamedImport(node) {
        var _a;
        if (!((_a = node.importClause) === null || _a === void 0 ? void 0 : _a.namedBindings) || !typescript_1.default.isNamedImports(node.importClause.namedBindings)) {
            return false;
        }
        const namedBindings = node.importClause.namedBindings;
        for (const item of namedBindings.elements) {
            this.pushItem(node, item.name, (0, util_cjs_1.nameToString)(item.propertyName || item.name));
        }
        return true;
    }
    /** import * as all from  */
    _addNamespaceImport(node) {
        var _a;
        if (!((_a = node.importClause) === null || _a === void 0 ? void 0 : _a.namedBindings) || !typescript_1.default.isNamespaceImport(node.importClause.namedBindings)) {
            return false;
        }
        const namedBindings = node.importClause.namedBindings;
        this.pushItem(node, namedBindings.name, '*');
        return true;
    }
    pushItem(node, identifier, stringifyName) {
        const symbol = this.checker.getSymbolAtLocation(identifier);
        if (symbol) {
            this._length++;
            // printMyDiagnostic(node, '%s imported as %s', stringifyName, identifier.getText());
            this.idRegistry.set(identifier, { node, identifier, stringifyName });
            this.unusedIds.add(identifier);
        }
        else {
            // printMyDiagnostic(node, 'imported identifier "%s" has no symbol', node.getText());
        }
    }
    findDeclaration(decl) {
        let ret;
        const checkSame = (id) => {
            if (id && this.unusedIds.has(id)) {
                return id;
            }
            return undefined;
        };
        if (typescript_1.default.isImportSpecifier(decl)) {
            // import {a} || import { a as b }
            return checkSame(decl.name);
        }
        else if (typescript_1.default.isNamespaceImport(decl)) {
            // import * as a
            return checkSame(decl.name);
        }
        else if (typescript_1.default.isImportClause(decl)) {
            // import def
            return checkSame(decl.name);
        }
        return ret;
    }
    found(id) {
        const info = this.idRegistry.get(id);
        if (!info) {
            (0, dumpNode_cjs_1.dumpNode)([...this.idRegistry.keys(), id]);
            throw new Error('program error.');
        }
        // console.log('!!!! %s (%s) used, is a value', info.stringifyName, info.node.getText());
        this.unusedIds.delete(id);
        return info;
    }
    listNotFound() {
        const ret = [];
        for (const id of this.unusedIds.values()) {
            ret.push(this.idRegistry.get(id));
        }
        return ret;
    }
}
exports.ImportIdentifierCollection = ImportIdentifierCollection;
//# sourceMappingURL=importIds.helper.js.map