"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.resolveTypescriptModule = void 0;
const path_1 = require("path");
const resolve_simulate_cjs_1 = require("./resolve.simulate.cjs");
/**
 * resolve import info from node_modules
 * @param packageJsonPath CURRENT project's package.json
 */
function resolveTypescriptModule(node, packageJsonPath) {
    const importPath = node.moduleSpecifier.text;
    const { packageName, filePath } = splitPackageName(importPath);
    // console.log('---------------- %s [%s]', importPath, basename(node.getSourceFile().fileName));
    const result = (0, resolve_simulate_cjs_1.resolveModule)('module', (0, path_1.resolve)(packageJsonPath, '../node_modules', packageName, 'package.json'), filePath, true);
    if (result) {
        const id = (0, path_1.relative)((0, path_1.resolve)(packageJsonPath, '../node_modules'), result).replace(/^\.\//, '');
        return {
            nodeResolve: id,
            specifier: importPath,
            type: "module" /* module */,
            sourceKind: "external" /* external */,
            fsPath: result,
        };
    }
    const result2 = (0, resolve_simulate_cjs_1.resolveModule)('commonjs', (0, path_1.resolve)(packageJsonPath, '../node_modules', packageName, 'package.json'), filePath, true);
    if (result2) {
        const id = (0, path_1.relative)((0, path_1.resolve)(packageJsonPath, '../node_modules'), result2).replace(/^\.\//, '');
        return {
            nodeResolve: id,
            specifier: importPath,
            type: "commonjs" /* commonjs */,
            sourceKind: "external" /* external */,
            fsPath: result2,
        };
    }
    return { specifier: importPath, type: "missing" /* missing */ };
}
exports.resolveTypescriptModule = resolveTypescriptModule;
function splitPackageName(path) {
    let i1 = path.indexOf('/');
    if (i1 === -1)
        return { packageName: path, filePath: '.' };
    if (path.startsWith('@')) {
        i1 = path.indexOf('/', i1 + 1);
        if (i1 === -1)
            return { packageName: path, filePath: '.' };
    }
    return { packageName: path.slice(0, i1), filePath: path.slice(i1 + 1) };
}
//# sourceMappingURL=resolve.node_modules.js.map