import { relative, resolve } from 'path';
import { resolveModule } from "./resolve.simulate.js";
/**
 * resolve import info from node_modules
 * @param packageJsonPath CURRENT project's package.json
 */
export function resolveTypescriptModule(node, packageJsonPath) {
    const importPath = node.moduleSpecifier.text;
    const { packageName, filePath } = splitPackageName(importPath);
    // console.log('---------------- %s [%s]', importPath, basename(node.getSourceFile().fileName));
    const result = resolveModule('module', resolve(packageJsonPath, '../node_modules', packageName, 'package.json'), filePath, true);
    if (result) {
        const id = relative(resolve(packageJsonPath, '../node_modules'), result).replace(/^\.\//, '');
        return {
            nodeResolve: id,
            specifier: importPath,
            type: "module" /* module */,
            sourceKind: "external" /* external */,
            fsPath: result,
        };
    }
    const result2 = resolveModule('commonjs', resolve(packageJsonPath, '../node_modules', packageName, 'package.json'), filePath, true);
    if (result2) {
        const id = relative(resolve(packageJsonPath, '../node_modules'), result2).replace(/^\.\//, '');
        return {
            nodeResolve: id,
            specifier: importPath,
            type: "commonjs" /* commonjs */,
            sourceKind: "external" /* external */,
            fsPath: result2,
        };
    }
    return { specifier: importPath, type: "missing" /* missing */ };
}
function splitPackageName(path) {
    let i1 = path.indexOf('/');
    if (i1 === -1)
        return { packageName: path, filePath: '.' };
    if (path.startsWith('@')) {
        i1 = path.indexOf('/', i1 + 1);
        if (i1 === -1)
            return { packageName: path, filePath: '.' };
    }
    return { packageName: path.slice(0, i1), filePath: path.slice(i1 + 1) };
}
//# sourceMappingURL=resolve.node_modules.js.map