import { builtinModules } from 'module';
import { relative } from 'path';
export function getImportedName(node) {
    return node.moduleSpecifier.text;
}
function _(node) {
    return typeof node === 'string' ? node : node.moduleSpecifier.text;
}
export function isImportNodeBuiltins(node) {
    const importPath = _(node);
    if (builtinModules) {
        return builtinModules.includes(importPath);
    }
    try {
        return !require.resolve(importPath).includes('/');
    }
    catch {
        return false;
    }
}
export function isImportFromNodeModules(node) {
    const importPath = _(node);
    // only when path is not relative
    if (importPath.startsWith('./') || importPath.startsWith('../'))
        return false;
    // TODO: tsconfig resolve
    return true;
}
export function createDiagnosticMissingImport(node) {
    return `(${nodeDiagnosticPosition(node)}) missing imported "${getImportedName(node)}".`;
}
/**
 * Create something like "xxx/yyy/zzz.ts:111:222"
 */
export function nodeDiagnosticPosition(node) {
    let posPart = '';
    try {
        const lines = node
            .getSourceFile()
            .getText()
            .slice(0, node.getStart() + 1)
            .split('\n');
        posPart = ':' + lines.length;
        const last = lines.pop().length;
        posPart += ':' + last;
    }
    catch { }
    const filePos = relative(process.cwd(), node.getSourceFile().fileName);
    return `${filePos}${posPart}`;
}
/**
 * split "@some/package/file.js" into "@some/package" and "file.js"
 */
export function splitPackageName(path) {
    let i1 = path.indexOf('/');
    if (i1 === -1)
        return { packageName: path, filePath: '.' };
    if (path.startsWith('@')) {
        i1 = path.indexOf('/', i1 + 1);
        if (i1 === -1)
            return { packageName: path, filePath: '.' };
    }
    return { packageName: path.slice(0, i1), filePath: path.slice(i1 + 1) };
}
//# sourceMappingURL=transform.modules.js.map