"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.testProjectFile = void 0;
const path_1 = require("path");
const fs_extra_1 = require("fs-extra");
/**
 * check an import statement, check if included in program, not an external/module file
 * @param debug logger
 * @param source which file contains this import
 * @param node a statement to check
 * @param program if set, limit project files by typescript, not only filesystem
 */
function testProjectFile(debug, source, node, program) {
    const importPath = node.moduleSpecifier.text;
    if (!importPath.startsWith('./') && !importPath.startsWith('../'))
        return false;
    if (importPath.endsWith('.js') || importPath.endsWith('.jsx')) {
        debug('import from %s, should not have extension.', importPath);
        return false;
    }
    const dir = (0, path_1.dirname)(source);
    for (const ext of ['.ts', '.tsx', '.js']) {
        const fp = (0, path_1.resolve)(dir, importPath + ext);
        if (program) {
            if (program.getSourceFile(fp)) {
                return true;
            }
        }
        else {
            if ((0, fs_extra_1.existsSync)(fp)) {
                return true;
            }
        }
    }
    debug('  ! imported file not in program');
    debug('      from:    %s', source);
    debug('      require: %s', importPath);
    return false;
}
exports.testProjectFile = testProjectFile;
//# sourceMappingURL=transform.project.js.map